from ..read_credentials import read_credentials
from ..save_credentials import save_credentials
from ..get_jwt import get_jwt
from ..save_jwt import save_jwt


def authenticate(site_url, username=None, password=None):
    if username is None and password is None:
        credentials = read_credentials(site_url)
        username = credentials["username"]
        password = credentials["password"]
    else:
        save_credentials(site_url, username, password)
    save_jwt(site_url, get_jwt(site_url, username, password))



from .read_jwt import read_jwt


def get_authentication_header(site_url):
    jwt = read_jwt(site_url)
    token = jwt.get("token", None)
    if token is None:
        raise AttributeError("There is no authentication token.")
    return {"Authorization": "Bearer " + token}



import click

from ....library.reports.commands.create_report import create_report

EXCEPTIONS_EXPECTED = (RuntimeError, FileNotFoundError, AttributeError)


@click.command(name="create")
def create_report_command():
    try:
        report_file = create_report()
        click.secho("Created report successfully.", fg="green", bold=True)
        click.secho(report_file, fg="green")
    except EXCEPTIONS_EXPECTED as exc:
        click.secho("Failed to create report.", fg="red", bold=True)
        click.secho(str(exc))



import maya

from .get_posts import get_posts


def get_posts_for_dates(site_url, start_date, duration_name="months", duration_value=1):
    # start date
    start_date = maya.when(start_date)
    end_date = start_date.add(**{duration_name: duration_value})
    # end date
    # get the posts
    posts_all = []
    posts_selected = []
    pages_tried_max = 100
    done = False
    for i in list(range(1, pages_tried_max + 1)):
        if done is True:
            continue
        try:
            posts = get_posts(site_url, page_number=i)
            posts_all.extend(posts)
        except Exception as exc:
            done = True
    for post in posts_all:
        post_date = maya.when(post["date"], timezone="UTC")
        if post_date >= start_date and post_date <= end_date:
            posts_selected.append(post)
    return posts_selected



import click

from .commands.authenticate import authenticate
from .commands.create_post import create_post


@click.group(name="wp")
def main():
    pass


main.add_command(authenticate)
main.add_command(create_post)



WORDPRESS_API_URI = "wp-json/wp/v2"

WORDPRESS_POSTS_URI = "{}/posts".format(WORDPRESS_API_URI)

WORDPRESS_TAGS_URI = "{}/tags".format(WORDPRESS_API_URI)

WORDPRESS_CATEGORIES_URI = "{}/categories".format(WORDPRESS_API_URI)

WORDPRESS_MEDIA_URI = "{}/media".format(WORDPRESS_API_URI)

WORDPRESS_JWT_URI = "wp-json/jwt-auth/v1"

DEFAULT_REQUEST_TIMEOUT = 10.0

DEFAULT_REQUEST_HEADERS = {
    "Content-Type": "application/json",
    "Accept": "application/json",
}

DEFAULT_CONFIG_FILE_NAME = "birdhouse-toolbox.json"

HTTP_SCHEMES = ("http://", "https://")

HTTP_METHODS_ALLOWED = ("get", "post", "put", "delete", "options", "head")

CONTENT_TYPES_ALLOWED = {
    "png": "image/png",
    "jpg": "image/jpeg",
    "jpeg": "image/jpeg",
    "gif": "image/gif",
}



from .get_post_content import get_post_content
from .get_header_html import get_header_html
from .get_footer_html import get_footer_html
from .get_form_html import get_form_html


def create_post_content(site_url, content_html):
    html_template = get_post_content(site_url)
    header_html = get_header_html(html_template)
    footer_html = get_footer_html(html_template)
    form_html = get_form_html(html_template)
    return "{}{}{}{}".format(header_html, content_html, form_html, footer_html)



import os
import click

from ....library.wordpress.commands.create_post import create_post as fn

EXCEPTIONS = (RuntimeError, AttributeError, FileNotFoundError)


@click.command(name="create")
@click.option("--title", required=True, help="The title of the post.")
@click.option("--content", required=True, help="The post content.")
@click.option(
    "--status",
    default="publish",
    required=False,
    help="May be either 'draft' or 'publish'.",
)
@click.option("--tag", multiple=True, required=False, help="The tags for the post.")
@click.option(
    "--category", multiple=True, required=False, help="The category of the post."
)
@click.option("--media", required=False, help="The path to a media file.")
@click.pass_obj
def create_post(options, title, content, status, tag, category, media):
    try:
        response = fn(
            site_url=options.url,
            title=title,
            content=content,
            status=status,
            tags=tag,
            categories=category,
            timeout=options.timeout,
            media=media,
        )
        click.secho("Post creation succeeded.", fg="green", bold=True)
    except EXCEPTIONS as exc:
        click.secho("Post creation failed.", fg="red", bold=True)
        click.secho(str(exc), fg="red")



import os

from ..request import request
from ..make_url import make_url
from ...settings import WORDPRESS_MEDIA_URI, CONTENT_TYPES_ALLOWED
from .get_authentication_header import get_authentication_header

INVALID_CONTENT_TYPE_MESSAGE = "Image must be one of {}"


def create_featured_media(site_url, file_path):
    if not os.path.exists(file_path):
        raise FileNotFoundError("Media file not found.")
    file_name = os.path.basename(file_path)
    ext = file_name.split(".")[-1]
    if ext not in CONTENT_TYPES_ALLOWED:
        raise RuntimeError(
            INVALID_CONTENT_TYPE_MESSAGE.format(", ".join(CONTENT_TYPES_ALLOWED.keys()))
        )
    headers = get_authentication_header(site_url)
    headers.update(
        {
            "Cache-Control": "no-cache",
            "Content-Disposition": "attachment; filename={}".format(file_name),
            "Content-Type": CONTENT_TYPES_ALLOWED[ext],
        }
    )
    url = make_url(site_url, WORDPRESS_MEDIA_URI)
    with open(file_path, "rb") as f:
        return request("post", url, data=f, headers=headers)



import os
from slugify import slugify

from ..get_authentication_header import get_authentication_header
from ..create_post_content import create_post_content
from ..get_or_create_tags import get_or_create_tags
from ..get_or_create_categories import get_or_create_categories
from ..create_featured_media import create_featured_media
from ...request import request
from ...make_url import make_url
from ...read_file import read_file
from ....settings import WORDPRESS_POSTS_URI

# TODO: Instead of using last post, allow getting a post by ID or date, instead.

# TODO: We only want to copy published posts, so we should also check for "Draft"
# status, before choosing a post to duplicate.


def create_post(site_url, title, content, status, tags, categories, media, timeout):
    html_markup = content
    if os.path.isfile(content) and content.endswith(".html"):
        html_markup = read_file(content)
    url = make_url(site_url, WORDPRESS_POSTS_URI)
    last_post = request("get", url, timeout=timeout)[0]
    featured_media = None
    if media is not None:
        featured_media = create_featured_media(site_url, media)["id"]
    return request(
        "post",
        url,
        timeout=timeout,
        headers=get_authentication_header(site_url),
        data={
            "slug": slugify(title),
            "status": status,
            "title": title,
            "content": create_post_content(site_url, html_markup),
            "tags": [x["id"] for x in get_or_create_tags(site_url, tags)],
            "categories": [
                x["id"] for x in get_or_create_categories(site_url, categories)
            ],
            "featured_media": featured_media,
            "meta": last_post["meta"],
            "template": last_post["template"],
        },
    )



def get_tag_html(html, opening_tag, closing_tag):
    opening_tag_start = html.find(opening_tag)
    opening_tag_end = opening_tag_start + len(opening_tag)
    closing_tag_start = html.find(closing_tag)
    closing_tag_end = closing_tag_start + len(closing_tag)
    return html[opening_tag_start:closing_tag_end]



import os
from io import open


def read_file(path):
    if not os.path.isfile(path):
        raise FileNotFoundError("The file does not exist. ({})".format(path))
    with open(path, "r", encoding="utf-8") as f:
        return f.read()



from ..request import request
from ..make_url import make_url
from ...settings import WORDPRESS_JWT_URI


def get_jwt(site_url, username, password):
    url = make_url(site_url, WORDPRESS_JWT_URI, "token")
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    data = {"username": username, "password": password}
    return request("post", url, headers=headers, data=data)



import requests
import json

from ..settings import (
    DEFAULT_REQUEST_TIMEOUT,
    DEFAULT_REQUEST_HEADERS,
    HTTP_METHODS_ALLOWED,
)

INVALID_METHOD_MESSAGE = "Method must be one of: {}".format(
    ", ".join(list(HTTP_METHODS_ALLOWED))
)
BAD_REQUEST_MESSAGE = "{status_code}: {url}\n\n{message}"
REQUEST_TIMEOUT_MESSAGE = "{url} timed out after {timeout} seconds"


def request(
    method, url, params={}, data={}, headers={}, timeout=DEFAULT_REQUEST_TIMEOUT
):
    method = method.lower()
    try:
        fn = getattr(requests, method)
    except AttributeError:
        raise RuntimeError(INVALID_METHOD_MESSAGE)

    final_headers = {}
    final_headers.update(DEFAULT_REQUEST_HEADERS)
    final_headers.update(headers)

    try:
        if method in ("get", "options", "head", "delete"):
            response = fn(url, params=params, headers=final_headers, timeout=timeout)

        elif method in ("post", "patch", "put"):
            current_content_type = final_headers["Content-Type"]
            if current_content_type == "application/json":
                data = json.dumps(data)

            response = fn(
                url, params=params, data=data, headers=final_headers, timeout=timeout
            )

        else:
            raise RuntimeError(INVALID_METHOD_MESSAGE)

        status_code = response.status_code
        if not str(status_code).startswith("2"):
            raise RuntimeError(
                BAD_REQUEST_MESSAGE.format(
                    status_code=status_code, url=url, message=response.text
                )
            )

        return response.json()

    except requests.ConnectionError:
        raise RuntimeError(REQUEST_TIMEOUT_MESSAGE.format(url=url, timeout=timeout))



import click

from .options import Options
from .wordpress.main import main as wordpress_group
from .analytics.main import main as analytics_group
from .reports.main import main as reports_group
from ..settings import DEFAULT_REQUEST_TIMEOUT


@click.group()
@click.option("--url")
@click.option("--timeout", default=DEFAULT_REQUEST_TIMEOUT, required=False)
@click.pass_context
def main(ctx, url, timeout):
    ctx.obj = Options(url, timeout)


main.add_command(wordpress_group)
main.add_command(analytics_group)
main.add_command(reports_group)



import json
from slugify import slugify

from ..request import request
from ...settings import WORDPRESS_TAGS_URI
from ..make_url import make_url
from .get_authentication_header import get_authentication_header


def create_tag(site_url, name):
    url = make_url(site_url, WORDPRESS_TAGS_URI)
    data = {"name": name, "slug": slugify(name)}
    headers = get_authentication_header(site_url)
    try:
        return request("post", url, data=data, headers=headers)
    except RuntimeError as exc:
        msg = str(exc)
        data_str = "".join(msg.split("\n")[2:])
        data = json.loads(data_str)
        if data["code"] == "term_exists":
            raise RuntimeError(
                "The term '{}' already exists as a category.".format(name)
            )
        raise exc


def get_tag(site_url, name):
    slug = slugify(name)
    url = make_url(site_url, WORDPRESS_TAGS_URI)
    response = request("get", url, params={"slug": slug})
    if len(response) > 1:
        raise RuntimeError("Found more than 1 tag for slug '{}'.".format(slug))
    return response[0]


def get_or_create_tag(site_url, name):
    tag = get_tag(site_url, name)
    if tag is None:
        tag = create_tag(site_url, name)
    return tag


def get_or_create_tags(site_url, tags):
    return [get_or_create_tag(site_url, x) for x in tags]



import click


from ....library.wordpress.commands.authenticate import authenticate as fn


@click.command(name="auth")
@click.option("--username", "-u")
@click.option("--password", "-p")
@click.pass_obj
def authenticate(options, username, password):
    try:
        fn(options.url, username, password)
        click.secho("Authentication succeeded.", fg="green", bold=True)
    except (AttributeError, RuntimeError) as exc:
        click.secho("Authentication failed.", fg="red", bold=True)
        click.secho(str(exc), fg="red")



from .get_tag_html import get_tag_html
from .get_header_html import get_header_html
from .get_footer_html import get_footer_html


def get_form_html(html):
    html = html.replace(get_header_html(html), "")
    html = html.replace(get_footer_html(html), "")
    noscript_html = get_tag_html(
        html,
        "<noscript class=\"ninja-forms-noscript-message\">",
        "</noscript>"
    )
    html = html.replace(noscript_html, "")
    div_html = get_tag_html(html, "<div>", "</div>")
    html = html.replace(div_html, "")
    script_html = get_tag_html(html, "<script>", "</script>")
    html = html.replace(script_html, "")
    return "{}{}{}".format(noscript_html, div_html, script_html)



import os
import json
from json import JSONDecodeError

from ..read_config import read_config
from ...settings import DEFAULT_CONFIG_FILE_NAME

MISSING_CONFIG_MESSAGE = "No wordpress config for {site_url}"
MISSING_CREDENTIALS_MESSAGE = "No wordpress credentials for {site_url}"


def read_credentials(site_url, config_file_name=DEFAULT_CONFIG_FILE_NAME):
    config = read_config(site_url, config_file_name)
    wordpress = config.get("wordpress", None)
    if wordpress is None:
        raise AttributeError(MISSING_CONFIG_MESSAGE.format(site_url=site_url))
    credentials = wordpress.get("credentials", None)
    if credentials is None:
        raise AttributeError(MISSING_CREDENTIALS_MESSAGE.format(site_url=site_url))
    return credentials



from ..make_url import make_url
from ..request import request
from ...settings import WORDPRESS_POSTS_URI, DEFAULT_REQUEST_TIMEOUT


def get_post_content(site_url, post_number=0):
    url = make_url(site_url, WORDPRESS_POSTS_URI)
    response = request("get", url)
    content = response[post_number]["content"]["rendered"]
    return content



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages
from setup_utils import read, read_section

PACKAGE_NAME = "birdhouse-toolbox"
SOURCE_DIR_NAME = "birdhouse_toolbox"
HOMEPAGE_URL = "https://www.alexseitsinger.com/packages/python/{}".format(PACKAGE_NAME)
GITHUB_URL = "https://github.com/alexseitsinger/{}".format(PACKAGE_NAME)
README_NAME = "README.md"

setup(
    name=PACKAGE_NAME,
    version=read(("src", SOURCE_DIR_NAME, "__init__.py"), "__version__"),
    description=read_section((README_NAME,), "Description", (0,)),
    long_description=read((README_NAME,)),
    long_description_content_type="text/markdown",
    author="Alex Seitsinger",
    author_email="software@alexseitsinger.com",
    url=HOMEPAGE_URL,
    install_requires=["requests", "click", "python-slugify", "maya"],
    entry_points={"console_scripts": ["bht={}.cli:main".format(SOURCE_DIR_NAME)]},
    package_dir={"": "src"},
    packages=find_packages("src", exclude=["tests"]),
    include_package_data=True,
    license="BSD 2-Clause License",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Communications",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
    project_urls={
        "Documentation": HOMEPAGE_URL,
        "Source": GITHUB_URL,
        "Tracker": "{}/issues".format(GITHUB_URL),
    },
)



from ..make_url import make_url
from ...settings import WORDPRESS_POSTS_URI
from ..request import request


def get_posts(site_url, page_number=1, per_page=100):
    url = make_url(site_url, WORDPRESS_POSTS_URI)
    params = {"per_page": per_page, "page": page_number}
    return request("get", url, params=params)



import os
import json
from json import JSONDecodeError

from .read_file import read_file
from ..settings import DEFAULT_CONFIG_FILE_NAME


def read_config(site_url, config_file_name=DEFAULT_CONFIG_FILE_NAME):
    config_file_path = os.path.join(os.path.expanduser("~"), config_file_name)
    if not os.path.exists(config_file_path):
        with open(config_file_path, "w") as f:
            f.write("{}")
    try:
        data = json.loads(read_file(config_file_path))
    except JSONDecodeError:
        data = {}
    if site_url not in data:
        raise AttributeError("There is no config for {}.".format(site_url))
    return data[site_url]



import os
import re
import itertools

from ..read_file import read_file


def to_html(tag_name, content):
    opening_tag = "<{}>".format(tag_name)
    closing_tag = "</{}>".format(tag_name)
    content = content.strip()
    return "{}{}{}".format(opening_tag, content, closing_tag)


def is_list_item(text):
    m = re.match(r"^[\d\w]\.", text)
    try:
        m.group()
        return True
    except AttributeError:
        return False


def create_item_list(sections, index):
    item_list = []
    start = index + 2
    end = len(sections) - 1
    items = sections[start:end]
    for item in items:
        if not len(item.strip()):
            break
        item_index = sections.index(item)
        if is_list_item(item):
            item_list.append(create_item_list(sections, item_index))
        else:
            item_list.append(item_index)
    return item_list


def create_html_list(sections, item_list, tag_name="ol"):
    first = item_list[0]
    last = item_list[-1]
    opening_tag = "<{}>".format(tag_name)
    closing_tag = "</{}>".format(tag_name)
    sections[first] = opening_tag + sections[first]
    sections[last] = sections[last] + closing_tag


def format_as_html(text):
    if os.path.isfile(text):
        text = read_file(text)
    sections = text.strip().split("\n")
    title_index = 0
    title = sections.pop(title_index)
    cta_index = -3
    cta = sections.pop(cta_index)
    item_lists = []
    paragraph_list = []
    for section in sections:
        if len(section.strip()):
            index = sections.index(section)
            if section.endswith(":"):
                item_lists.append(create_item_list(sections, index))
            indexes = list(itertools.chain.from_iterable(item_lists))
            if index not in indexes:
                paragraph_list.append(index)
    for index in paragraph_list:
        sections[index] = to_html("p", sections[index])
    for item_list in item_lists:
        for index in item_list:
            sections[index] = to_html("li", sections[index])
    for item_list in item_lists:
        create_html_list(sections, item_list)
    sections.insert(cta_index + 1, to_html("h3", cta))
    sections.insert(title_index, to_html("h2", title))
    return "".join(sections)
